@@ -16,6 +16,7 @@ class RailtieTest < ActiveSupport::TestCase
     @app.config.eager_load = false
     @app.config.logger = Logger.new(nil)
     @app.config.secret_key_base = ('x' * 30)
+    @app.config.active_support.cache_format_version = Rails::VERSION::STRING.to_f
   end
 
   test 'GlobalID.app for Blog::Application defaults to blog' do
@@ -65,18 +66,6 @@ class RailtieTest < ActiveSupport::TestCase
     assert_equal @app.message_verifier(:signed_global_ids).generate(message), signed_message
   end
 
-  test 'SignedGlobalID.verifier defaults to nil when secret_key_base is not present' do
-    original_env, Rails.env = Rails.env, 'production'
-
-    begin
-      @app.config.secret_key_base = nil
-      @app.initialize!
-      assert_nil SignedGlobalID.verifier
-    ensure
-      Rails.env = original_env
-    end
-  end
-
   test 'SignedGlobalID.verifier can be set with config.global_id.verifier =' do
     custom_verifier = @app.config.global_id.verifier = ActiveSupport::MessageVerifier.new('muchSECRETsoHIDDEN', serializer: SERIALIZER)
     @app.initialize!
